From cf2dae1def02d33912d59462164db6c6f875bf08 Mon Sep 17 00:00:00 2001
From: Juan Jose Garcia Ripoll <jjgarcia@jjgr-2.local>
Date: Sat, 17 Apr 2010 21:12:33 +0200
Subject: [PATCH] Implement a declarative option :TESTS for running tests.

---
 asdf.lisp |   29 +++++++++++++++++++++++++++++
 1 files changed, 29 insertions(+), 0 deletions(-)

diff --git a/asdf.lisp b/asdf.lisp
index e1c5621..a8911c8 100644
--- a/asdf.lisp
+++ b/asdf.lisp
@@ -928,6 +928,7 @@ actually-existing directory."
    (maintainer :accessor system-maintainer :initarg :maintainer)
    (licence :accessor system-licence :initarg :licence
             :accessor system-license :initarg :license)
+   (tests :accessor system-tests :initarg :tests)
    (source-file :reader system-source-file :initarg :source-file
                 :writer %set-system-source-file)))
 
@@ -1745,10 +1746,38 @@ recursive calls to traverse.")
 (defmethod perform ((operation test-op) (c component))
   nil)
 
+(defun parse-function-name (f-designator)
+  (handler-case
+      (with-standard-io-syntax
+        (read-from-string (string f-designator)))
+    (error (c) (sysdef-error "~@<invalid function designator ~A~@:>"
+                             f-designator))))
+
+(defun test-system-designator (c)
+  (and (consp c)
+       (string-equal (first c) "system")
+       (= (length c) 2)
+       (second c)))
+
+(defmethod perform ((o test-op) (c system))
+  (loop for item in (system-tests c)
+     do (cond ((typep item '(or string symbol))
+               (funcall (parse-function-name item)))
+              ((test-system-designator item))
+              (t (sysdef-error "~@<invalid argument to a system :tests ~A~@:>"
+                               item)))))
+
 (defmethod operation-done-p ((operation test-op) (c system))
   "Testing a system is _never_ done."
   nil)
 
+(defmethod component-depends-on ((o test-op) (c system))
+  (let ((tests (system-tests c)))
+    (loop for item in tests
+       when (and (consp item)
+                 (string-equal (first item) "system"))
+       collect `(test-op ,(second item)))))
+
 (defmethod component-depends-on :around ((o test-op) (c system))
   (cons `(load-op ,(component-name c)) (call-next-method)))
 
-- 
1.6.6.1

